@@ -1,4 +1,6 @@
 /*
+ * $Id: aecho.c,v 1.9 2009-10-14 01:38:28 didg Exp $
+ *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
  *
@@ -25,30 +27,62 @@
  * AppleTalk Echo Protocol Client
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/time.h>
-#include <sys/errno.h>
-
-#include <netatalk/endian.h>
-#include <netatalk/at.h>
-#include <atalk/aep.h>
-#include <atalk/nbp.h>
-#include <atalk/ddp.h>
+#include <errno.h>
 
 #include <signal.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif /* HAVE_NETDB_H */
 
-struct sockaddr_at	target;
-int			s, nsent = 0, nrecv = 0;
-long			totalms = 0, minms = -1, maxms = -1;
+#include <netatalk/endian.h>
+#include <netatalk/at.h>
+#include <atalk/compat.h>
+#include <atalk/aep.h>
+#include <atalk/nbp.h>
+#include <atalk/netddp.h>
+#include <atalk/ddp.h>
+#include <atalk/util.h>
 
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif ibm032 _IBMR2
-aep_send()
+/* FIXME/SOCKLEN_T: socklen_t is a unix98 feature */
+#ifndef SOCKLEN_T
+#define SOCKLEN_T unsigned int
+#endif /* ! SOCKLEN_T */
+
+static struct sockaddr_at	target;
+static int			sock;
+static unsigned int		nsent = 0, nrecv = 0;
+static time_t			totalms = 0, minms = -1, maxms = -1;
+static unsigned int     	pings = 0;
+
+static void usage(char *);
+
+static void done(int sig _U_)
+{
+    if ( nsent) {
+	printf( "\n----%d.%d AEP Statistics----\n",
+		ntohs( target.sat_addr.s_net ), target.sat_addr.s_node );
+	printf( "%d packets sent, %d packets received, %d%% packet loss\n",
+		nsent, nrecv, (( nsent - nrecv ) * 100 ) / nsent );
+	if ( nrecv ) {
+	    printf( "round-trip (ms)  min/avg/max = %ld/%ld/%ld\n",
+		    minms, totalms / nrecv, maxms );
+	}	
+    }
+    exit( 0 );
+}
+  
+static void aep_send(int sig _U_)
 {
     struct timeval	tv;
     char		*p, buf[ 1024 ];
@@ -57,7 +91,7 @@ aep_send()
     p = buf;
     *p++ = DDPTYPE_AEP;
     *p++ = AEPOP_REQUEST;
-    bcopy( &seq, p, sizeof( unsigned int ));
+    memcpy( p, &seq, sizeof( unsigned int ));
     p += sizeof( unsigned int );
     seq++;
 
@@ -65,96 +99,107 @@ aep_send()
 	perror( "gettimeofday" );
 	exit( 1 );
     }
-    bcopy( &tv, p, sizeof( struct timeval ));
+    memcpy( p, &tv, sizeof( struct timeval ));
     p += sizeof( struct timeval );
 
-    if ( sendto( s, buf, p - buf, 0, &target,
+    if ( netddp_sendto( sock, buf, p - buf, 0, (struct sockaddr *) &target,
 	    sizeof( struct sockaddr_at )) < 0 ) {
 	perror( "sendto" );
 	exit( 1 );
     }
     nsent++;
+    if (pings && nsent > pings) done(0);
 }
 
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif ibm032 _IBMR2
-done()
-{
-    if ( nsent > 0 ) {
-	printf( "\n----%d.%d AEP Statistics----\n",
-		ntohs( target.sat_addr.s_net ), target.sat_addr.s_node );
-	printf( "%d packets sent, %d packets received, %d%% packet loss\n",
-		nsent, nrecv, (( nsent - nrecv ) * 100 ) / nsent );
-	if ( nrecv > 0 ) {
-	    printf( "round-trip (ms)  min/avg/max = %d/%d/%d\n",
-		    minms, totalms / nrecv, maxms );
-	}	
-    }
-    exit( 0 );
-}
-
-main( ac, av )
-    int		ac;
-    char	**av;
+int main(int ac, char **av)
 {
     struct servent	*se;
     struct sigaction	sv;
     struct itimerval	it;
-    struct sockaddr_at	sat;
+    struct sockaddr_at	sat, saddr;
     struct timeval	tv, atv;
     struct nbpnve	nn;
     char		*obj = NULL, *type = "Workstation", *zone = "*";
-    int			satlen, cc;
+    int			cc;
+    SOCKLEN_T		satlen;
     unsigned int	seq;
-    long		ms;
+    time_t		ms;
     char		buf[ 1024 ], *p;
     unsigned char	port;
-    extern int		errno;
 
-    if ( ac != 2 ) usage( av[ 0 ] );
+    extern char		*optarg;
+    extern int		optind;
+  
+    memset(&saddr, 0, sizeof(saddr));
+    while (( cc = getopt( ac, av, "c:A:" )) != EOF ) {
+	switch ( cc ) {
+	case 'A':
+	    if (!atalk_aton(optarg, &saddr.sat_addr)) {
+	        fprintf(stderr, "Bad address.\n");
+		exit(1);
+	    }
+	    break;
 
+	  case 'c' :
+	    pings = atoi( optarg );
+	    break;
+
+	default :
+	    usage( av[ 0 ] );
+	    exit( 1 );
+	}
+    }
+    if ( ac - optind != 1 ) {
+	usage( av[ 0 ] );
+	exit( 1 );
+    }
+    
     /*
      * Save the port, since nbp_lookup calls getservbyname() to get the
      * nbp port.
      */
-    if (( se = getservbyname( "echo", "ddp" )) == NULL ) {
-	fprintf( stderr, "Unknown service.\n" );
-	exit( 1 );
-    }
-    port = ntohs( se->s_port );
+    if (( se = getservbyname( "echo", "ddp" )) == NULL ) 
+       port = 4;
+    else
+       port = ntohs( se->s_port );
 
-    bzero( &target, sizeof( struct sockaddr_at ));
+    memset( &target, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    target.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     target.sat_family = AF_APPLETALK;
-    if ( !atalk_aton( av[ 1 ], &target.sat_addr )) {
-	if ( nbp_name( av[ 1 ], &obj, &type, &zone ) || !obj ) {
-	    fprintf( stderr, "Bad name: %s\n", av[ 1 ] );
+    if ( !atalk_aton( av[ optind ], &target.sat_addr )) {
+	if ( nbp_name( av[ optind ], &obj, &type, &zone ) || !obj ) {
+	    fprintf( stderr, "Bad name: %s\n", av[ optind ] );
 	    exit( 1 );
 	}
-	if ( nbp_lookup( obj, type, zone, &nn, 1 ) <= 0 ) {
-	    fprintf( stderr, "Can't find: %s\n", av[ 1 ] );
+	if ( nbp_lookup( obj, type, zone, &nn, 1, &saddr.sat_addr) <= 0 ) {
+	    fprintf( stderr, "Can't find: %s\n", av[ optind ] );
 	    exit( 1 );
 	}
-	bcopy( &nn.nn_sat, &target, sizeof( struct sockaddr_at ));
+	memcpy( &target, &nn.nn_sat, sizeof( struct sockaddr_at ));
     }
     target.sat_port = port;
 
-    if (( s = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
-	perror( "socket" );
-	exit( 1 );
-    }
+    if ((sock = netddp_open(saddr.sat_addr.s_net || saddr.sat_addr.s_node ? 
+			 &saddr : NULL, NULL)) < 0) {
+       perror("ddp_open");
+       exit(1);
+    }        
 
     sv.sa_handler = aep_send;
-    sv.sa_mask = sigmask( SIGINT );
-    sv.sa_flags = 0;
+    sigemptyset( &sv.sa_mask );
+    sigaddset( &sv.sa_mask, SIGINT );
+    sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGALRM, &sv, (struct sigaction *)0 ) < 0 ) {
 	perror( "sigaction" );
 	exit( 1 );
     }
 
     sv.sa_handler = done;
-    sv.sa_mask = sigmask( SIGALRM );
-    sv.sa_flags = 0;
+    sigemptyset( &sv.sa_mask );
+    sigaddset( &sv.sa_mask, SIGALRM );
+    sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGINT, &sv, (struct sigaction *)0 ) < 0 ) {
 	perror( "sigaction" );
 	exit( 1 );
@@ -172,8 +217,9 @@ main( ac, av )
 
     for (;;) {
 	satlen = sizeof( struct sockaddr_at );
-	if (( cc = recvfrom( s, buf, sizeof( buf ), 0, &sat,
-		&satlen )) < 0 ) {
+	if (( cc = netddp_recvfrom( sock, buf, sizeof( buf ), 0, 
+				    (struct sockaddr *) &sat,
+				    &satlen )) < 0 ) {
 	    if ( errno == EINTR ) {
 		errno = 0;
 		continue;
@@ -191,9 +237,9 @@ main( ac, av )
 	    perror( "gettimeofday" );
 	    exit( 1 );
 	}
-	bcopy( p, &seq, sizeof( unsigned int ));
+	memcpy( &seq, p, sizeof( unsigned int ));
 	p += sizeof( unsigned int );
-	bcopy( p, &atv, sizeof( struct timeval ));
+	memcpy( &atv, p, sizeof( struct timeval ));
 	nrecv++;
 	ms = ( tv.tv_sec - atv.tv_sec ) * 1000 +
 		( tv.tv_usec - atv.tv_usec ) / 1000;
@@ -204,14 +250,15 @@ main( ac, av )
 	if ( ms < minms || minms == -1 ) {
 	    minms = ms;
 	}
-	printf( "%d bytes from %u.%u: aep_seq=%d. time=%d. ms\n",
+	printf( "%d bytes from %u.%u: aep_seq=%d. time=%ld. ms\n",
 		cc, ntohs( sat.sat_addr.s_net ), sat.sat_addr.s_node,
 		seq, ms );
+        if (pings && seq + 1 >= pings) done(0);
     }
 }
 
-usage( av0 )
+static void usage( char * av0 )
 {
-    fprintf( stderr, "Usage:\t%s ( addr | nbpname )\n", av0 );
+    fprintf( stderr, "usage:\t%s [-A source address ] [-c count] ( addr | nbpname )\n", av0 );
     exit( 1 );
 }
