@@ -1,5 +1,5 @@
 /*
- * $Id: asingle.h,v 1.2 2001/06/29 14:14:46 rufustfirefly Exp $
+ * $Id: asingle.h,v 1.4 2010-01-27 21:27:53 didg Exp $
  */
 
 #ifndef _ASINGLE_H
@@ -12,6 +12,6 @@ int single_open(char *singlefile, int flags, struct FHeader *fh, int options);
 int single_close(int readflag);
 int single_header_read(struct FHeader *fh, int version);
 int single_header_test(void);
-int single_read(int fork, char *buffer, int length);
+ssize_t single_read(int fork, char *buffer, size_t length);
 
 #endif /* _ASINGLE_H */
