@@ -27,7 +27,7 @@
 #define AFPD_DESKTOP_H 1
 
 #include <sys/cdefs.h>
-#include "globals.h"
+#include <atalk/globals.h>
 #include "volume.h"
 
 struct savedt {
