@@ -1,5 +1,5 @@
 /*
- * $Id: uams_krb4.c,v 1.6.10.1 2003/09/11 23:49:30 bfernhomberg Exp $
+ * $Id: uams_krb4.c,v 1.10 2009-10-15 11:39:48 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -86,7 +86,7 @@ struct ClearToken {
 
 #ifdef KRB
 
-static __inline__ void lcase( p )
+static void lcase( p )
     char	*p;
 {
     for (; *p; p++ ) {
@@ -97,7 +97,7 @@ static __inline__ void lcase( p )
     return;
 }
 
-static __inline__ void ucase( p )
+static void ucase( p )
     char	*p;
 {
     for (; *p; p++ ) {
@@ -121,8 +121,8 @@ static __inline__ void ucase( p )
 #define KRB4CMD_SKIP	11
 
 static int krb4_login(void *obj, struct passwd **uam_pwd,
-		      char *ibuf, int ibuflen,
-		      char *rbuf, int *rbuflen )
+		      char *ibuf, size_t ibuflen,
+		      char *rbuf, size_t *rbuflen )
 {
     char		*p;
     char		*username;
@@ -130,7 +130,8 @@ static int krb4_login(void *obj, struct passwd **uam_pwd,
     u_int16_t		len;
     KTEXT_ST		tkt;
     static AUTH_DAT	ad;
-    int			rc, ulen, proto;
+    int			rc, proto;
+    size_t		ulen;
     char		inst[ 40 ], princ[ 40 ];
 
     if (uam_afpserver_option(obj, UAM_OPTION_USERNAME, &username, &ulen) < 0)
@@ -233,7 +234,7 @@ static int krb4_login(void *obj, struct passwd **uam_pwd,
 #ifdef AFS
     if ( setpag() < 0 ) {
 	*rbuflen = 0;
-	LOG(log_error, logtype_default, "krb_login: setpag: %m" );
+	LOG(log_error, logtype_default, "krb_login: setpag: %s", strerror(errno) );
 	return( AFPERR_BADUAM );
     }
 #endif /*AFS*/
@@ -252,8 +253,8 @@ static int krb4_action( void *v1, void *v2, const int i )
    with non-16bit short's and non-32bit int's
 */
 static int krb4_logincont(void *obj, struct passwd **uam_pwd,
-			  char *ibuf, int ibuflen,
-			  char *rbuf, int *rbuflen)
+			  char *ibuf, size_t ibuflen,
+			  char *rbuf, size_t *rbuflen)
 {
     static struct passwd	*pwd;
     KTEXT_ST		tkt;
@@ -326,7 +327,7 @@ static int krb4_logincont(void *obj, struct passwd **uam_pwd,
 	    vi.out_size = sizeof( buf );
 
 	    if ( pioctl( 0, VIOCSETTOK, &vi, 0 ) < 0 ) {
-		LOG(log_error, logtype_default, "krb4_logincont: pioctl: %m" );
+		LOG(log_error, logtype_default, "krb4_logincont: pioctl: %s", strerror(errno) );
 		*rbuflen = 0;
 		return( AFPERR_BADUAM );
 	    }
@@ -444,7 +445,7 @@ static int krb4_logincont(void *obj, struct passwd **uam_pwd,
 		    vi.out = buf;
 		    vi.out_size = sizeof( buf );
 		    if ( pioctl( 0, VIOCSETTOK, &vi, 0 ) < 0 ) {
-			LOG(log_error, logtype_default, "krb4_logincont: pioctl: %m" );
+			LOG(log_error, logtype_default, "krb4_logincont: pioctl: %s", strerror(errno) );
 			return( AFPERR_BADUAM );
 		    }
 		    /* FALL THROUGH */
@@ -567,8 +568,8 @@ static void authenticate(cells,name,passwd)
 
 #if defined( UAM_AFSKRB ) && defined( AFS )
 static int afskrb_login(void *obj, struct passwd *uam_pwd,
-			char *ibuf, int ibuflen, 
-			char *rbuf, int *rbuflen )
+			char *ibuf, size_t ibuflen, 
+			char *rbuf, size_t *rbuflen )
 {
     KTEXT_ST	authent, rpkt;
     CREDENTIALS	cr;
@@ -609,7 +610,7 @@ static int afskrb_login(void *obj, struct passwd *uam_pwd,
     p = rbuf;
     if ( validseskey == 0 ) {
 	if ( setpag() < 0 ) {
-	    LOG(log_error, logtype_default, "krb_login: setpag: %m" );
+	    LOG(log_error, logtype_default, "krb_login: setpag: %s", strerror(errno) );
 	    return AFPERR_BADUAM;
 	}
 	krb_set_tkt_string(( tktfile = mktemp( _PATH_AFPTKT )));
@@ -626,7 +627,7 @@ static int afskrb_login(void *obj, struct passwd *uam_pwd,
 	}
 
 	if ( unlink( tktfile ) < 0 ) {
-	    LOG(log_error, logtype_default, "krb_login: unlink %s: %m", tktfile );
+	    LOG(log_error, logtype_default, "krb_login: unlink %s: %s", tktfile, strerror(errno) );
 	    return ( AFPERR_BADUAM );
 	}
 
@@ -671,8 +672,8 @@ static int afskrb_login(void *obj, struct passwd *uam_pwd,
 }
 
 static int afskrb_logincont(void *obj, struct passwd *uam_pwd,
-			    char *ibuf, int ibuflen, 
-			    char *rbuf, int *rbuflen )
+			    char *ibuf, size_t ibuflen, 
+			    char *rbuf, size_t *rbuflen )
 {
     CREDENTIALS		cr;
     struct ViceIoctl	vi;
@@ -737,12 +738,12 @@ static int afskrb_logincont(void *obj, struct passwd *uam_pwd,
     vi.out = buf;
     vi.out_size = sizeof( buf );
     if ( pioctl( 0, VIOCSETTOK, &vi, 0 ) < 0 ) {
-	LOG(log_error, logtype_default, "krb_logincont: pioctl: %m" );
+	LOG(log_error, logtype_default, "krb_logincont: pioctl: %s", strerror(errno) );
 	return ( AFPERR_BADUAM );
     }
 
     if ( unlink( tktfile ) < 0 ) {
-	LOG(log_error, logtype_default, "krb_logincont: %s: %m", tktfile );
+	LOG(log_error, logtype_default, "krb_logincont: %s: %s", tktfile, strerror(errno) );
 	return ( AFPERR_BADUAM );
     }
 
