@@ -1,4 +1,6 @@
 /*
+ * $Id: asp_write.c,v 1.4 2009-10-22 12:35:39 franklahm Exp $
+ *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
  *
@@ -21,55 +23,51 @@
  *	netatalk@itd.umich.edu
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <string.h>
 #include <sys/types.h>
-#include <sys/time.h>
 #include <sys/uio.h>
 #include <netatalk/endian.h>
 #include <netatalk/at.h>
 #include <atalk/atp.h>
 #include <atalk/asp.h>
 
-char		rbuf[ 8 ][ ATP_MAXDATA ];
-struct iovec	iov[ 8 ] = {
-    { rbuf[ 0 ], 0 },
-    { rbuf[ 1 ], 0 },
-    { rbuf[ 2 ], 0 },
-    { rbuf[ 3 ], 0 },
-    { rbuf[ 4 ], 0 },
-    { rbuf[ 5 ], 0 },
-    { rbuf[ 6 ], 0 },
-    { rbuf[ 7 ], 0 },
-};
+#if defined(BSD) || defined(BSD4_3)
+#define memmove(a, b, n)   bcopy((b), (a), (n))
+#endif /* BSD || BSD4_3 */
 
-asp_wrtcont( asp, buf, buflen )
-    ASP			asp;
-    char		*buf;
-    int			*buflen;
+int asp_wrtcont(ASP asp, char *buf, size_t *buflen)
 {
+    struct iovec	iov[ ASP_MAXPACKETS ];
     struct atp_block	atpb;
-    char		*p;
-    int			iovcnt = 8;
-    u_short		blen, seq;
-    u_char		oport;
+    char	        *p;
+    int			iovcnt = ASP_MAXPACKETS;
+    u_int16_t		blen, seq;
+    u_int8_t		oport;
 
     p = buf;
     *p++ = ASPFUNC_WRTCONT;
     *p++ = asp->asp_sid;
     seq = htons( asp->asp_seq );
-    bcopy( &seq, p, sizeof( u_short ));
-    p += sizeof( u_short );
-    blen = htons( *buflen );
-    bcopy( &blen, p, sizeof( u_short ));
+    memcpy( p, &seq, sizeof(seq));
+    p += sizeof(seq);
+    blen = htons(*buflen);
+    memcpy( p, &blen, sizeof(blen));
+    p += sizeof(blen);
 
-    for ( iovcnt = 0; iovcnt < 8; iovcnt++ ) {
-	iov[ iovcnt ].iov_len = ATP_MAXDATA;
+    for ( iovcnt = 0; iovcnt < ASP_MAXPACKETS; iovcnt++ ) {
+        iov[iovcnt].iov_base = buf + iovcnt*ASP_CMDMAXSIZ;
+	iov[ iovcnt ].iov_len = ASP_CMDMAXSIZ;
     }
 
     oport = asp->asp_sat.sat_port;
     atpb.atp_saddr = &asp->asp_sat;
     atpb.atp_saddr->sat_port = asp->asp_wss;
     atpb.atp_sreqdata = buf;
-    atpb.atp_sreqdlen = 6;			/* Hard coding numbers? */
+    atpb.atp_sreqdlen = p - buf;
     atpb.atp_sreqto = 2;
     atpb.atp_sreqtries = 5;
 
@@ -77,6 +75,8 @@ asp_wrtcont( asp, buf, buflen )
 	asp->asp_sat.sat_port = oport;
 	return( -1 );
     }
+    asp->write_count += atpb.atp_sreqdlen;
+
     atpb.atp_rresiov = iov;
     atpb.atp_rresiovcnt = iovcnt;
     if ( atp_rresp( asp->asp_atp, &atpb ) < 0 ) {
@@ -86,20 +86,15 @@ asp_wrtcont( asp, buf, buflen )
 
     asp->asp_sat.sat_port = oport;
 
+    /* get rid of the 4-byte headers */
     p = buf;
     for ( iovcnt = 0; iovcnt < atpb.atp_rresiovcnt; iovcnt++ ) {
-	bcopy( iov[ iovcnt ].iov_base + 4, p, iov[ iovcnt ].iov_len - 4 );
-	p += ( iov[ iovcnt ].iov_len - 4 );
+   	memmove(p, (char *) iov[ iovcnt ].iov_base + ASP_HDRSIZ, 
+		iov[ iovcnt ].iov_len - ASP_HDRSIZ );
+	p += ( iov[ iovcnt ].iov_len - ASP_HDRSIZ );
     }
-    *buflen = p - buf;
-    return( 0 );
-}
 
-asp_wrtreply( asp, result, buf, buflen )
-    ASP		asp;
-    int		result;
-    char	*buf;
-    int		buflen;
-{
-    return( asp_cmdreply( asp, result, buf, buflen ));
+    *buflen = p - buf;
+    asp->read_count += *buflen;
+    return 0;
 }
