@@ -1,5 +1,5 @@
 /*
- * $Id: cnid_cdb_get.c,v 1.1.4.2.2.1 2005/01/30 20:56:22 didg Exp $
+ * $Id: cnid_cdb_get.c,v 1.5 2009-10-29 13:38:16 didg Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -11,7 +11,7 @@
 
 /* Return CNID for a given did/name. */
 cnid_t cnid_cdb_get(struct _cnid_db *cdb, const cnid_t did, char *name,
-                const int len)
+                const size_t len)
 {
     char start[CNID_DID_LEN + MAXPATHLEN + 1], *buf;
     CNID_private *db;
@@ -46,7 +46,7 @@ cnid_t cnid_cdb_get(struct _cnid_db *cdb, const cnid_t did, char *name,
 
     memcpy(&id, data.data, sizeof(id));
 #ifdef DEBUG
-    LOG(log_info, logtype_default, "cnid_get: Returning CNID for %u, name %s as %u",
+    LOG(log_debug9, logtype_default, "cnid_get: Returning CNID for %u, name %s as %u",
         ntohl(did), name, ntohl(id));
 #endif
     return id;
