@@ -1,5 +1,5 @@
 /*
- * $Id: cnid_tdb_resolve.c,v 1.1.2.1 2003/09/09 16:42:21 didg Exp $
+ * $Id: cnid_tdb_resolve.c,v 1.4 2009-11-22 14:14:05 franklahm Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -11,7 +11,7 @@
 #include "cnid_tdb.h"
 
 /* Return the did/name pair corresponding to a CNID. */
-char *cnid_tdb_resolve(struct _cnid_db *cdb, cnid_t * id, void *buffer, u_int32_t len)
+char *cnid_tdb_resolve(struct _cnid_db *cdb, cnid_t * id, void *buffer, size_t len)
 {
     struct _cnid_tdb_private *db;
     TDB_DATA key, data;      
@@ -25,8 +25,8 @@ char *cnid_tdb_resolve(struct _cnid_db *cdb, cnid_t * id, void *buffer, u_int32_
     if (data.dptr) 
     {
         if (data.dsize < len && data.dsize > sizeof(cnid_t)) {
-            memcpy(id, (char *)data.dptr + TDB_DEVINO_LEN, sizeof(cnid_t));
-            strcpy(buffer, (char *)data.dptr + TDB_HEADER_LEN);
+            memcpy(id, (char *)data.dptr + +CNID_DID_OFS, sizeof(cnid_t));
+            strcpy(buffer, (char *)data.dptr + CNID_NAME_OFS);
             free(data.dptr);
             return buffer;
         }
